@@ -5,8 +5,8 @@ end
 
 require 'minitest/autorun'
 
-class TestCase < MiniTest::Unit::TestCase
+class TestCase < MiniTest::Test
   def self.test(name, &block)
     define_method("test_#{name.gsub(/\W/, '_')}", &block) if block
   end
-end
\ No newline at end of file
+end
